﻿using Connected.Events;
using Microsoft.AspNetCore.SignalR.Client;

namespace Connected.Net;

internal class EventConnection : IDisposable, IAsyncDisposable
{
	private HubConnection? _connection;

	public EventConnection(IRemoteProviderService remote, IEventService events)
	{
		Remote = remote;
		Events = events;
	}

	private bool IsDisposed { get; set; }
	protected HubConnection? Connection => _connection;
	private IRemoteProviderService Remote { get; }
	private IEventService Events { get; }

	public virtual async Task Initialize()
	{
		if (Connection is not null)
		{
			if (Connection.State != HubConnectionState.Disconnected)
				await Connection.StopAsync();

			await Connection.DisposeAsync();
		}

		_connection = new HubConnectionBuilder()
			.WithUrl($"{Remote.ProvideBaseUrl()}/events")
			.WithAutomaticReconnect()
			.Build();

		_connection.Reconnecting += OnReconnecting;
		_connection.Reconnected += OnReconnected;
		_connection.Closed += OnClosed;

		_connection.On<MessageAcknowledgeArgs, RemoteEventArgs>("Notify", async (a, e) =>
		{
			await _connection.SendAsync("Acknowledge", a);

			if (Events is EventService service)
				service.Trigger(e);
		});
	}

	private async Task OnClosed(Exception? arg)
	{
		await Task.CompletedTask;
	}

	private async Task OnReconnected(string? arg)
	{
		await Task.CompletedTask;
	}

	private async Task OnReconnecting(Exception? arg)
	{
		await Task.CompletedTask;
	}

	public async Task Connect()
	{
		if (_connection is null)
			await Initialize();

		if (Connection is null)
			throw new NullReferenceException(nameof(Connection));

		if (Connection.State != HubConnectionState.Disconnected)
			return;

		await Connection.StartAsync();
	}

	public async Task Disconnect()
	{
		if (Connection is null)
			return;

		if (Connection.State == HubConnectionState.Disconnected)
			await Connection.StopAsync();
	}

	public async Task Notify<TArgs>(string method, TArgs args)
	{
		if (Connection is null)
			return;

		// TODO: Buffer message if connection is not opened and send it later.
		if (Connection.State != HubConnectionState.Connected)
			return;

		if (Connection.State == HubConnectionState.Connected)
			await Connection.SendAsync(method, args);
	}

	private void Dispose(bool disposing)
	{
		if (!IsDisposed)
		{
			if (disposing)
			{
				if (_connection is not null)
				{
					var task = _connection.DisposeAsync();

					if (!task.IsCompleted)
						task.GetAwaiter().GetResult();

					_connection = null;
				}
			}

			IsDisposed = true;
		}
	}
	public void Dispose()
	{
		Dispose(true);
		GC.SuppressFinalize(this);
	}

	public async ValueTask DisposeAsync()
	{
		if (_connection is not null)
		{
			await _connection.DisposeAsync();
			_connection = null;
		}

		GC.SuppressFinalize(this);
	}
}
