﻿using System.Globalization;

namespace Connected;

public static class DateExtensions
{
	public static DateTime FromUtc(DateTime value, TimeZoneInfo timeZone)
	{
		if (value == DateTime.MinValue || value.Kind != DateTimeKind.Utc)
			return value;

		if (timeZone is null || timeZone == TimeZoneInfo.Utc)
			return value;
		else
			return TimeZoneInfo.ConvertTimeFromUtc(value, timeZone);
	}

	public static DateTime ToUtc(DateTime value, TimeZoneInfo timeZone)
	{
		if (value == DateTime.MinValue || value.Kind == DateTimeKind.Utc)
			return value;

		if (timeZone is null || timeZone == TimeZoneInfo.Utc)
			return value;
		else
			return TimeZoneInfo.ConvertTimeToUtc(value, timeZone);
	}

	public static DateTimeOffset FromUtc(DateTimeOffset value, TimeZoneInfo timeZone)
	{
		if (value == DateTimeOffset.MinValue)
			return value;

		if (timeZone is null || timeZone == TimeZoneInfo.Utc)
			return new DateTimeOffset(new DateTime(value.UtcDateTime.Ticks, DateTimeKind.Utc));
		else
		{
			var offset = FromUtc(value.UtcDateTime, timeZone);

			return new DateTimeOffset(offset, timeZone.GetUtcOffset(value.UtcDateTime));
		}
	}

	public static DateTimeOffset ToUtc(DateTimeOffset value, TimeZoneInfo timeZone)
	{
		if (value == DateTimeOffset.MinValue || value.Offset == TimeSpan.Zero)
			return value;

		if (timeZone is null || timeZone == TimeZoneInfo.Utc)
			return value;
		else
			return new DateTimeOffset(value.UtcDateTime);
	}

	public static DateTime StartOfWeek(this DateTime value, DayOfWeek? firstDayOfWeek)
	{
		firstDayOfWeek ??= CultureInfo.CurrentUICulture.DateTimeFormat.FirstDayOfWeek;

		var diff = (int)(7 + (value.DayOfWeek - firstDayOfWeek)) % 7;

		if (value.Year == 1 && value.Month == 1 && (value.Day - diff) < 1)
			return value.Date;

		return value.AddDays(-1 * diff).Date;
	}

	public static string ToIsoDateString(this DateTime value)
	{
		return $"{value.Year:D4}-{value.Month:D2}-{value.Day:D2}";
	}

	public static string ToIsoDateString(this DateTime? value)
	{
		if (value is null)
			return null;

		return $"{value.Value.Year:D4}-{value.Value.Month:D2}-{value.Value.Day:D2}";
	}

	public static DateTime StartOfMonth(this DateTime value, CultureInfo culture)
	{
		var month = culture.Calendar.GetMonth(value);
		var year = culture.Calendar.GetYear(value);

		return culture.Calendar.ToDateTime(year, month, 1, 0, 0, 0, 0);
	}

	public static DateTime EndOfMonth(this DateTime value, CultureInfo culture)
	{
		var month = culture.Calendar.GetMonth(value);
		var year = culture.Calendar.GetYear(value);
		var days = culture.Calendar.GetDaysInMonth(year, month);

		return culture.Calendar.ToDateTime(year, month, days, 0, 0, 0, 0);
	}
}
